'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
//region Imports
var path = require('path');
var _ = require('lodash');
//endregion

// Babel logging.
function logFileHelpers() {
  var through = require('through2');
  var gutil = require('gulp-util');

  return through.obj(function (file, enc, cb) {
    var src = file.history[0];
    var dest = file.history[1];
    // TODO(vjpr): When generating source maps this should print src as generated.
    var relPath = function relPath(f) {
      return path.relative('.', f);
    };
    gutil.log('Compiling (Babel):', gutil.colors.red(relPath(src) + ' -> ' + relPath(dest)));
    //gutil.log(file.babel.usedHelpers)
    cb(null, file);
  });
}

function onError() {
  var notify = require('gulp-notify');
  return notify.onError('<%= error.message %>');
  //return notify.onError((e) => {
  //  console.log(e)
  //  const filename = path.relative('.', e.fileName)
  //  const moduleName = path.basename(process.cwd())
  //  return `${moduleName} - ${filename}\n ${e.message}`
  //  // TODO(vjpr): Not working.
  //  //return {
  //  //  title: 'babel compile error',
  //  //  message: `[${moduleName}] ${filename}`,
  //  //}
  //})
}

exports.default = function (gulp, opts) {

  ////////
  // This is shared.
  var transformAsyncToModuleMethod = require('babel-plugin-transform-async-to-module-method');
  var asyncPlugin = [transformAsyncToModuleMethod, {
    module: 'bluebird',
    method: 'coroutine'
  }];
  var addModuleExports = require('babel-plugin-add-module-exports');
  var transformRuntime = require('babel-plugin-transform-runtime');
  var externalHelpers = require('babel-plugin-external-helpers-2');
  var babelDefaults = {
    plugins: [externalHelpers, asyncPlugin, transformRuntime, addModuleExports],
    presets: ['es2015', 'stage-0', 'react']
  };
  ////////

  _.defaults(opts, {
    srcGlob: '',
    shouldCompileTests: false,
    destDir: '',
    testsGlob: '',
    babel: babelDefaults
  });

  gulp.task('lib:compile', function () {
    var gulpif = require('gulp-if');
    var changed = require('gulp-changed');
    var ignore = require('gulp-ignore');
    var babel = require('gulp-babel');
    var sourcemaps = require('gulp-sourcemaps');
    var cwd = require('cwd');

    return gulp.src(opts.srcGlob).pipe(gulpif(gulp.isWatching, changed(opts.destDir))).pipe(gulpif(!opts.shouldCompileTests, ignore.exclude(opts.testsGlob))).pipe(sourcemaps.init()).pipe(babel(opts.babel)).on('error', onError())
    // Source maps work with IntelliJ debugger using Node.js 0.12.7.
    .pipe(sourcemaps.write('.', {
      includeContent: false,
      sourceRoot: function sourceRoot(file) {
        return path.join(path.relative(file.path, cwd()), opts.srcDir);
      }
    })).pipe(gulp.dest(opts.destDir)).pipe(logFileHelpers());
  });

  gulp.task('lib:clean', function () {
    var del = require('del');
    var gutil = require('gulp-util');

    gutil.log('Cleaning:', gutil.colors.red(opts.destDir));
    return del(opts.destGlob);
  });

  gulp.task('lib:watch', gulp.series('lib:clean', 'lib:compile', function watcher() {
    var gutil = require('gulp-util');

    // Used by `compile` task to check whether we should only compile changed files.
    gulp.isWatching = true;

    gulp.watch(opts.srcGlob, gulp.series('lib:compile')).on('change', function (evt) {
      gutil.log('---');
      gutil.log('File changed:', gutil.colors.red(evt));
    });
  }));
};

module.exports = exports['default'];